﻿using Microsoft.Xna.Framework.Content;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace RhythmBullet.Zer01HD.Utilities.ContentSystem
{
    class ContentSystem
    {
        Thread thread;
        internal volatile bool loading;
        readonly ContentManager contentManager;
        readonly Queue<ContentLoad> queue;
        Dictionary<string, IDisposable> assets;
        readonly Dictionary<Type, IContentResolver> contentResolver;

        public ContentSystem(ContentManager contentManager)
        {
            this.contentManager = contentManager;
            assets = new Dictionary<string, IDisposable>();
            queue = new Queue<ContentLoad>();
            contentResolver = new Dictionary<Type, IContentResolver>();
        }

        private void Load(string assetName, Type type)
        {
            IContentResolver handler = contentResolver[type];
            string path = handler.Load(assetName);
            assets.Add(assetName, contentManager.Load<IDisposable>(path));
        }

        private void LoadBatch()
        {
            while (queue.Count != 0)
            {
                lock (queue)
                {
                    ContentLoad content = queue.Dequeue();
                    Load(content.assetName, content.type);
                }
            }
        }

        T get<T>(string assetName)
        {
            lock(queue)
            {
                return (T)assets[assetName];
            }
        }

        void Queue(string assetName, Type type)
        {
            lock (queue)
            {
                if (!assets.ContainsKey(assetName))
                {
                    queue.Enqueue(new ContentLoad(assetName, type));
                }
            }
        }

        /// <summary>
        /// Called whenever a batch of assets should be loaded from the queue. Safe to call once every frame.
        /// </summary>
        void Update()
        {
            if (queue.Count > 0)
            {
                if (thread == null || !thread.IsAlive)
                {
                    ThreadStart threadStart = new ThreadStart(LoadBatch);
                    thread = new Thread(threadStart);
                }
            }
        }

        void UnloadAll()
        {
            foreach (KeyValuePair<string, IDisposable> asset in assets)
            {
                asset.Value.Dispose();
                assets.Remove(asset.Key);
            }
        }
    }
}
